//
//     MINOTAUR -- It's only 1/2 bull
//
//     (C)opyright 2010 - 2012 The MINOTAUR Team.
//

/**
 * \file CutManager.h
 * \brief Manages addition and deletion of cuts to problem.
 * \author Ashutosh Mahajan, Argonne National Laboratory
 */

#ifndef MINOTAURCUTMANAGER_H
#define MINOTAURCUTMANAGER_H

#include "Types.h"

namespace Minotaur {

class CutPool;
class Environment;
class Logger;
class Problem;
class Solution;
typedef boost::shared_ptr<CutPool> CutPoolPtr;
typedef boost::shared_ptr<Environment> EnvironmentPtr;
typedef boost::shared_ptr<Logger> LoggerPtr;
typedef boost::shared_ptr<Problem> ProblemPtr;
typedef boost::shared_ptr<Solution> SolutionPtr;

/**
 * The CutManager class is meant to manage the cutting planes generated by
 * different cut generators and handlers. 
 */
class CutManager {

public:
  /// Default constructor.
  CutManager();

  /// Constructor that loads the relaxation problem. 
  CutManager(ProblemPtr p);

  /// Destroy.
  ~CutManager();

  /// Add a cut to be managed by the cut manager
  /// If directToRel == true, force it to go to relaxation
  /// If neverDelete == true, cut should never be removed from pool
  void addCut(ConstraintPtr cons, Bool directToRel, Bool neverDelete);

  /// Checks cuts in pool and possibly adds them to relaxation
  /// sol is the solution we would like to cut off
  void checkCuts(SolutionPtr sol);

  /// Updates information about the cuts based on the given relaxation
  /// solution
  /// Also removes cuts from relaxation
  /// Sol is the solution of the last relaxation solve
  /// TODO: Also need to add the basis status info
  void updateCuts(SolutionPtr sol);

  /// Write to output.
  void write(std::ostream &out) const;

private:
  /// Pool of cuts.
  CutPoolPtr pool_;

  /// Environment.
  EnvPtr env_;

  /// Random vector to check for duplicacy.
  Double* hashVec_;

  /// For logging.
  LoggerPtr logger_;

  /// For logging.
  const static std::string me_;

  /// The relaxation problem that cuts are added to and deleted from.
  ProblemPtr p_;
};

typedef boost::shared_ptr<CutManager> CutManagerPtr;
}
#endif

// Local Variables: 
// mode: c++ 
// eval: (c-set-style "k&r") 
// eval: (c-set-offset 'innamespace 0) 
// eval: (setq c-basic-offset 2) 
// eval: (setq fill-column 78) 
// eval: (auto-fill-mode 1) 
// eval: (setq column-number-mode 1) 
// eval: (setq indent-tabs-mode nil) 
// End:
