// 
//     MINOTAUR -- It's only 1/2 bull
// 
//     (C)opyright 2008 - 2012 The MINOTAUR Team.
// 

#ifndef MINOTAURMULTILINEARFUNCTION_H
#define MINOTAURMULTILINEARFUNCTION_H

#include "NonlinearFunction.h"

namespace Minotaur {

  class SparseMatrix;
  class SymmetricSparseMatrix;

  typedef std::pair<Double, std::set<ConstVariablePtr> > multilinearTerm;
  typedef std::vector<multilinearTerm> multilinearTermContainer;
  typedef std::vector<multilinearTerm>::const_iterator constMultilinearTermContainerIterator;
  

  class MultilinearFunction : public NonlinearFunction {

  public:
    MultilinearFunction() { }
    ~MultilinearFunction() { }

    constMultilinearTermContainerIterator termsBegin() const { return terms_.begin(); }
    constMultilinearTermContainerIterator termsEnd() const { return terms_.end(); }

    void addTerm(multilinearTerm t) { terms_.push_back(t); }

    boost::shared_ptr<MultilinearFunction> clone() const;
    Double eval(const Double *x, Int *error);
    Double eval(const std::vector<Double> &x) const;
    void evalGrd(const std::vector<Double> &x, Double &c, std::vector<Double> &g) const;
    void evalGrd(const std::vector<Double> &x, Double &c, SparseMatrix &g) const;
    void evalHes(const std::vector<Double> &x, Double &l, SymmetricSparseMatrix &h) const;

    UInt getNumTerms() const { return(terms_.size()); }

    void display() ;


  private:
    multilinearTermContainer terms_;
  };

  typedef boost::shared_ptr<MultilinearFunction> MultilinearFunctionPtr;
  typedef boost::shared_ptr<const MultilinearFunction> ConstMultilinearFunctionPtr;
}
#endif

// Local Variables: 
// mode: c++ 
// eval: (c-set-style "k&r") 
// eval: (c-set-offset 'innamespace 0) 
// eval: (setq c-basic-offset 2) 
// eval: (setq fill-column 78) 
// eval: (auto-fill-mode 1) 
// eval: (setq column-number-mode 1) 
// eval: (setq indent-tabs-mode nil) 
// End:
