//
//     MINOTAUR -- It's only 1/2 bull
//
//     (C)opyright 2008 - 2012 The MINOTAUR Team.
//

/**
 * \file TransPoly.h
 * \brief Declare class for reformulating a polynomial problem.
 * \author Ashutosh Mahajan, Argonne National Laboratory
 */

#ifndef MINOTAURTRANSPOLY_H
#define MINOTAURTRANSPOLY_H

#include "Transformer.h"

namespace Minotaur {

class MultilinearHandler;
typedef boost::shared_ptr<MultilinearHandler> MultilinearHandlerPtr;

/**
 * \brief Base class for reformulating a mixed-integer polynomial problem
 * into one that has univariate (power) constraints and multilinear constraints. 
 */
class TransPoly : public Transformer {
public:

  /// Constructor.
  TransPoly();

  /// Constructor.
  TransPoly(EnvPtr env);

  ~TransPoly();

  SolutionPtr getSolOrig(ConstSolutionPtr sol);

  SolutionPtr getSolTrans(ConstSolutionPtr sol);

  void reformulate(ConstProblemPtr oldp, ProblemPtr &newp,
                   HandlerVector &handlers, Int &status);

  void refNonlinCons(ConstProblemPtr , ProblemPtr ) {};

  void refNonlinCons(ConstProblemPtr oldp, ProblemPtr newp,
                     HandlerVector &handlers);
    
private:
  MultilinearHandlerPtr mhandler_;
  void addPolyCons_(PolyFunPtr pf, ProblemPtr newp);
};
}

#endif

// Local Variables: 
// mode: c++ 
// eval: (c-set-style "k&r") 
// eval: (c-set-offset 'innamespace 0) 
// eval: (setq c-basic-offset 2) 
// eval: (setq fill-column 78) 
// eval: (auto-fill-mode 1) 
// eval: (setq column-number-mode 1) 
// eval: (setq indent-tabs-mode nil) 
// End:
